angular.module('Api', [])
  // Mobil részen Config
  .constant('ApiConfig', {
    url: 'http://mobil.eclickapi.com/hungaro_control/Api.json',
    debug: true,
    tokenName: 'token',
    userIdName: 'user_id',
    lastSyncName: 'lastSyncDate',
    authDataKey: 'userAuthData',
    loginRedirect: 'home.my_profile',
    logoutRedirect: 'game',
    loginAction: 'login',
    loginState: {
      stateName: 'login',
      url: '/login',
      templateUrl: 'index.html',
      controller: 'Login'
    }
  })
  // A szerveren lévő hiba kódok
  .constant('ApiConst', {
    ERROR_CODE_UNSUPPORTED_HTTP_METHOD: 1000,
    ERROR_CODE_UNDEFINED_ACTION: 1001,
    ERROR_CODE_UNDEFINED_FUNCTION: 1002,
    ERROR_CODE_MISSING_PARAMS: 1003,
    ERROR_CODE_UNAUTHORIZED: 1004,
    ERROR_CODE_NO_FB_REGISTRATION: 1122
  })
  /**
   * Run fügvény  itt lesz az allow
   * @TODO: Nincs még kész az allow
   */
  .run(function ($rootScope, AuthConfig, Api) {
    //var Api = new Api();
    $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {
      //console.log(toState);
      if (!AuthConfig.loggedIn && AuthConfig.allow.indexOf(toState.name) == -1) {
        //console.log('Not Allow');
        //event.preventDefault();
      } else {
        //console.log('Allow');
      }
    });
  })
  /**
   * URL Config beállítás
   * Alapértelmezett kezdő url beállítás
   * Login url beállítás
   */
  .config(function ($stateProvider, $urlRouterProvider, ApiConfig) {
    var loginStateName = ApiConfig.loginState.stateName;
    delete ApiConfig.loginState.stateName;

    $stateProvider
      .state(loginStateName, ApiConfig.loginState);

    $urlRouterProvider.otherwise(function () {
      return '/' + ApiConfig.loginAction;
    });
  })
  /**
   * Auth Config-ok
   */
  .value('AuthConfig', {
    allow: [
      'login',
      'logout'
    ],
    loggedIn: false
  })
  /**
   * Auth parancsok
   */
  .factory('Auth', ['Api', 'ApiConfig', '$state', 'AuthConfig', function (Api, ApiConfig, $state, AuthConfig) {
    //var Api = new Api();
    var self = this;
    //function Auth() {
    self.loginRedirect = ApiConfig.loginRedirect;
    self.logoutRedirect = ApiConfig.logoutRedirect;
    self.loginAction = ApiConfig.loginAction;
    //}

    self.allow = function () {
      for (var arg in arguments) {
        if (self.allow.indexOf(arg) > -1) {
          AuthConfig.allow.push(arg);
        }
      }
    };

    self.loggedIn = function (callback) {
      callback = (callback == undefined) ? false : callback;

      Api.get('loggedIn', null, function (response) {
        AuthConfig.loggedIn = response.data.success;
        if (callback) {
          callback(AuthConfig.loggedIn);
          return;
        }
      }, true);
    };

    self.logout = function (callback) {
      callback = (callback == undefined) ? false : callback;
      Api.get('logout', null, function (response) {
        window.localStorage.removeItem('Auth');
        Api.removeToken();
        Api.removeLastSyncDate();
        Api.removeUserId();
        AuthConfig.loggedIn = false;
        if (callback) {
          callback(response);
          return;
        } else {
          $state.go(self.logoutRedirect);
        }
      }, true);
    };

    self.login = function (authParams, callback) {
      callback = (callback == undefined) ? false : callback;
      Api.post('login', authParams, function (response) {
        if (response.data.success) {
          window.localStorage.setItem('Auth', response.data.data);
          Api.setToken(response.data.data.token);
          Api.setUserData(response.data.data);
          AuthConfig.loggedIn = true;
        } else {
          AuthConfig.loggedIn = false;
        }
        if (callback) {
          callback(response);
          return;
        } else {
          $state.go(self.loginRedirect);
        }
      }, false);
    };
    return self;
  }])
  /**
   * Api kezelő
   */
  .factory('Api', ['$http', 'ApiConfig', '$state', 'ApiConst', function ($http, ApiConfig, $state, ApiConst) {
    var self = this;

    //function Api() {
    self.url = ApiConfig.url;
    self.tokenName = ApiConfig.tokenName;
    self.userIdName = ApiConfig.userIdName;
    self.lastSyncName = ApiConfig.lastSyncName;
    self.authDataKey = ApiConfig.authDataKey;
    self.loginRedirect = ApiConfig.loginRedirect;
    self.logoutRedirect = ApiConfig.logoutRedirect;
    self.loginAction = ApiConfig.loginAction;
    //};

    self.errorCode = function (response) {
      var errorCode = response.data.errorCode;
      switch (errorCode) {
        case ApiConst.ERROR_CODE_UNAUTHORIZED:
          console.log('Nem vagy bejeletkezve!');
          $state.go(ApiConfig.loginAction);
          return false;
          break;
        case ApiConst.ERROR_CODE_UNSUPPORTED_HTTP_METHOD:
        case ApiConst.ERROR_CODE_UNDEFINED_ACTION:
        case ApiConst.ERROR_CODE_UNDEFINED_FUNCTION:
        case ApiConst.ERROR_CODE_MISSING_PARAMS:
          console.error("[Error Code: " + errorCode + "]\n" + response.data.message);
          return false;
          break;
        default:
          return true;
          break;
      }
    };

    self.request = function (method, action, params, callback, tokenExtend) {
      params = (params == undefined) ? {} : params;
      callback = (callback == undefined) ? false : callback;
      tokenExtend = (tokenExtend == undefined) ? false : tokenExtend;

      var params = angular.extend({
        action: action
      }, params);

      if (tokenExtend) {
        params = angular.extend(params, {
          token: self.getToken()
        });
      }

      var data = {
        method: method,
        url: self.url,
        responseType: 'json',
        params: params
      };

      if (ApiConfig.debug) {
        console.debug('Api request');
        console.dir(data);
      }

      $http(data).then(function (response) {
        if (self.errorCode(response)) {
          if (callback) {
            callback(response);
          }
        } else {
          if (callback) {
            callback(response);
          }
        }
      }, function (response) {
        console.warn(JSON.stringify(response));
        if (callback) {
          callback(false);
        }
      });
    };

    self.get = function (action, params, callback, tokenExtend) {
      self.request('GET', action, params, callback, tokenExtend);
    };

    self.post = function (action, params, callback, tokenExtend) {
      self.request('POST', action, params, callback, tokenExtend);
    };
    self.file = function (action, params, callback, tokenExtend) {
      params = (params == undefined) ? {} : params;
      callback = (callback == undefined) ? false : callback;
      tokenExtend = (tokenExtend == undefined) ? false : tokenExtend;

      var fd = new FormData();
      fd.append('action', action);
      for (var i in params) {
        fd.append(i, params[i]);
      }
      console.log(fd);
      if (tokenExtend) {
        fd.append('token', self.getToken());
      }


      if (ApiConfig.debug) {
        console.debug('Api request');
      }

      $http.post(self.url, fd, {
          transformRequest: angular.identity,
          headers: {'Content-Type': undefined}
        })
        .success(function (response) {

          if (callback) {
            callback(response);
          }

        })
        .error(function (response) {
          console.warn(JSON.stringify(response));
          if (callback) {
            callback(false);
          }
        });

    };

    self.setToken = function (value) {
      window.localStorage.setItem(self.tokenName, value);
    };

    self.getToken = function () {
      return window.localStorage.getItem(self.tokenName);
    };

    self.setUserData = function (value) {
      window.localStorage.setItem(self.authDataKey, JSON.stringify(value));
    };

    self.getUserData = function () {
      return JSON.parse(window.localStorage.getItem(self.authDataKey));
    };

    self.checkToken = function () {
      return (window.localStorage.getItem(self.tokenName) === null);
    };

    self.removeToken = function () {
      window.localStorage.removeItem(self.tokenName);
    };

    self.setUserId = function (value){
      window.localStorage.setItem(self.userIdName, value);
    };

    self.getUserId = function () {
      return window.localStorage.getItem(self.userIdName);
    };

    self.removeUserId = function () {
      window.localStorage.removeItem(self.userIdName);
    };

    self.setLastSyncDate = function (value){
      window.localStorage.setItem(self.lastSyncName, value);
    };

    self.getLastSyncDate = function () {
      return window.localStorage.getItem(self.lastSyncName);
    };

    self.removeLastSyncDate = function () {
      window.localStorage.removeItem(self.lastSyncName);
    };

    return self;
  }]);
