angular.module('ui.rCalendar').controller('ui.rCalendar.CalendarController', ['$scope', '$attrs', '$parse', '$interpolate', '$log', 'dateFilter', 'calendarConfig', function ($scope, $attrs, $parse, $interpolate, $log, dateFilter, calendarConfig) {
  'use strict';
  var self = this,
    ngModelCtrl = {$setViewValue: angular.noop}; // nullModelCtrl;

  // Configuration attributes
  angular.forEach(['formatDay', 'formatDayHeader', 'formatDayTitle', 'formatWeekTitle', 'formatMonthTitle', 'formatWeekViewDayHeader', 'formatHourColumn',
    'showWeeks', 'showEventDetail', 'startingDay', 'eventSource', 'queryMode'], function (key, index) {
    self[key] = angular.isDefined($attrs[key]) ? (index < 7 ? $interpolate($attrs[key])($scope.$parent) : $scope.$parent.$eval($attrs[key])) : calendarConfig[key];
  });
  $scope.today=new Date();
  $scope.$parent.$watch($attrs.eventSource, function (value) {
    self.onEventSourceChanged(value);
  });

  $scope.calendarMode = $scope.calendarMode || calendarConfig.calendarMode;
  if (angular.isDefined($attrs.initDate)) {
    self.currentCalendarDate = $scope.$parent.$eval($attrs.initDate);
  }
  if (!self.currentCalendarDate) {
    self.currentCalendarDate = new Date();
    if ($attrs.ngModel && !$scope.$parent.$eval($attrs.ngModel)) {
      $parse($attrs.ngModel).assign($scope.$parent, self.currentCalendarDate);
    }
  }

  self.init = function (ngModelCtrl_) {
    ngModelCtrl = ngModelCtrl_;
    ngModelCtrl.$render = function () {
      self.render();
    };
  };

  self.render = function () {
    if (ngModelCtrl.$modelValue) {
      var date = new Date(ngModelCtrl.$modelValue),
        isValid = !isNaN(date);

      if (isValid) {
        this.currentCalendarDate = date;
      } else {
        $log.error('"ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
      }
      ngModelCtrl.$setValidity('date', isValid);
    }
    this.refreshView();
  };

  self.refreshView = function () {
    if (this.mode) {
      this.range = this._getRange(this.currentCalendarDate);
      this._refreshView();
      this.rangeChanged();
    }
  };

  // Split array into smaller arrays
  self.split = function (arr, size) {
    var arrays = [];
    while (arr.length > 0) {
      arrays.push(arr.splice(0, size));
    }
    return arrays;
  };

  self.onEventSourceChanged = function (value) {
    self.eventSource = value;
    if (self._onDataLoaded) {
      self._onDataLoaded();
    }
  };

  $scope.move = function (direction) {
    var step = self.mode.step,
      currentCalendarDate = self.currentCalendarDate,
      year = currentCalendarDate.getFullYear() + direction * (step.years || 0),
      month = currentCalendarDate.getMonth() + direction * (step.months || 0),
      date = currentCalendarDate.getDate() + direction * (step.days || 0),
      firstDayInNextMonth;

    currentCalendarDate.setFullYear(year, month, date);
    if ($scope.calendarMode === 'month') {
      firstDayInNextMonth = new Date(year, month + 1, 1);
      if (firstDayInNextMonth.getTime() <= currentCalendarDate.getTime()) {
        self.currentCalendarDate = new Date(firstDayInNextMonth - 24 * 60 * 60 * 1000);
      }
    }
    ngModelCtrl.$setViewValue(self.currentCalendarDate);
    self.refreshView();
  };

  self.move = function (direction) {
    $scope.move(direction);
  };

  self.rangeChanged = function () {
    if (self.queryMode === 'local') {
      if (self.eventSource && self._onDataLoaded) {
        self._onDataLoaded();
      }
    } else if (self.queryMode === 'remote') {
      if ($scope.rangeChanged) {
        $scope.rangeChanged({
          startTime: this.range.startTime,
          endTime: this.range.endTime
        });
      }
    }
  };

  function overlap(event1, event2) {
    return !(event1.endIndex <= event2.startIndex || event2.endIndex <= event1.startIndex);
  }

  function calculatePosition(events) {
    var i,
      j,
      len = events.length,
      maxColumn = 0,
      col,
      isForbidden = new Array(len);

    for (i = 0; i < len; i += 1) {
      for (col = 0; col < maxColumn; col += 1) {
        isForbidden[col] = false;
      }
      for (j = 0; j < i; j += 1) {
        if (overlap(events[i], events[j])) {
          isForbidden[events[j].position] = true;
        }
      }
      for (col = 0; col < maxColumn; col += 1) {
        if (!isForbidden[col]) {
          break;
        }
      }
      if (col < maxColumn) {
        events[i].position = col;
      } else {
        events[i].position = maxColumn++;
      }
    }
  }

  function calculateWidth(orderedEvents) {
    var cells = new Array(24),
      event,
      index,
      i,
      j,
      len,
      eventCountInCell,
      currentEventInCell;

    //sort by position in descending order, the right most columns should be calculated first
    orderedEvents.sort(function (eventA, eventB) {
      return eventB.position - eventA.position;
    });
    for (i = 0; i < 24; i += 1) {
      cells[i] = {
        calculated: false,
        events: []
      };
    }
    len = orderedEvents.length;
    for (i = 0; i < len; i += 1) {
      event = orderedEvents[i];
      index = event.startIndex;
      while (index < event.endIndex) {
        cells[index].events.push(event);
        index += 1;
      }
    }

    i = 0;
    while (i < len) {
      event = orderedEvents[i];
      if (!event.overlapNumber) {
        var overlapNumber = event.position + 1;
        event.overlapNumber = overlapNumber;
        var eventQueue = [event];
        while ((event = eventQueue.shift())) {
          index = event.startIndex;
          while (index < event.endIndex) {
            if (!cells[index].calculated) {
              cells[index].calculated = true;
              if (cells[index].events) {
                eventCountInCell = cells[index].events.length;
                for (j = 0; j < eventCountInCell; j += 1) {
                  currentEventInCell = cells[index].events[j];
                  if (!currentEventInCell.overlapNumber) {
                    currentEventInCell.overlapNumber = overlapNumber;
                    eventQueue.push(currentEventInCell);
                  }
                }
              }
            }
            index += 1;
          }
        }
      }
      i += 1;
    }
  }

  self.placeEvents = function (orderedEvents) {
    calculatePosition(orderedEvents);
    calculateWidth(orderedEvents);
  };

  self.placeAllDayEvents = function (orderedEvents) {
    calculatePosition(orderedEvents);
  };
}])
