angular.module('ui.rCalendar').directive('dayview', ['dateFilter', '$timeout', function (dateFilter, $timeout) {
  'use strict';
  return {
    restrict: 'EA',
    replace: true,
    templateUrl: 'lib/calendar/views/day.html',
    require: '^calendar',
    link: function (scope, element, attrs, ctrl) {
      scope.formatHourColumn = ctrl.formatHourColumn;

      $timeout(function () {
        updateScrollGutter();
      });

      ctrl.mode = {
        step: {days: 1}
      };

      function updateScrollGutter() {
        var children = element.children();
        var allDayEventBody = children[0].children[1];
        var allDayEventGutterWidth = allDayEventBody.offsetWidth - allDayEventBody.clientWidth;
        var normalEventBody = children[1];
        var normalEventGutterWidth = normalEventBody.offsetWidth - normalEventBody.clientWidth;
        var gutterWidth = allDayEventGutterWidth || normalEventGutterWidth || 0;
        if (gutterWidth > 0) {
          if (allDayEventGutterWidth <= 0) {
            scope.allDayEventGutterWidth = gutterWidth;
          } else {
            scope.allDayEventGutterWidth = 0;
          }
          if (normalEventGutterWidth <= 0) {
            scope.normalGutterWidth = gutterWidth;
          } else {
            scope.normalGutterWidth = 0;
          }
        }
      }

      function createDateObjects(startTime) {
        var rows = [],
          time,
          currentHour = startTime.getHours(),
          currentDate = startTime.getDate();

        for (var hour = 0; hour < 24; hour += 1) {
          time = new Date(startTime.getTime());
          time.setHours(currentHour + hour);
          time.setDate(currentDate);
          rows.push({
            time: time
          });
        }
        return rows;

      }

      scope.select = function (selectedTime) {
        if (scope.timeSelected) {
          scope.timeSelected({selectedTime: selectedTime});
        }
      };

      ctrl._onDataLoaded = function () {
        var eventSource = ctrl.eventSource,
          len = eventSource ? eventSource.length : 0,
          startTime = ctrl.range.startTime,
          endTime = ctrl.range.endTime,
          timeZoneOffset = -new Date().getTimezoneOffset(),
          utcStartTime = new Date(startTime.getTime() + timeZoneOffset * 60 * 1000),
          utcEndTime = new Date(endTime.getTime() + timeZoneOffset * 60 * 1000),
          rows = scope.rows,
          allDayEvents = [],
          oneHour = 3600000,
          eps = 0.016,
          eventSet,
          normalEventInRange = false,
          hour;

        if (rows.hasEvent) {
          for (hour = 0; hour < 24; hour += 1) {
            if (rows[hour].events) {
              rows[hour].events = null;
            }
          }
          rows.hasEvent = false;
        }

        for (var i = 0; i < len; i += 1) {
          var event = eventSource[i];
          var eventStartTime = new Date(event.startTime);
          var eventEndTime = new Date(event.endTime);

          if (event.allDay) {
            if (eventEndTime <= utcStartTime || eventStartTime >= utcEndTime) {
              continue;
            } else {
              allDayEvents.push({
                event: event
              });
            }
          } else {
            if (eventEndTime <= startTime || eventStartTime >= endTime) {
              continue;
            } else {
              normalEventInRange = true;
            }

            var timeDifferenceStart;
            if (eventStartTime <= startTime) {
              timeDifferenceStart = 0;
            } else {
              timeDifferenceStart = (eventStartTime - startTime) / oneHour;
            }

            var timeDifferenceEnd;
            if (eventEndTime >= endTime) {
              timeDifferenceEnd = (endTime - startTime) / oneHour;
            } else {
              timeDifferenceEnd = (eventEndTime - startTime) / oneHour;
            }

            var startIndex = Math.floor(timeDifferenceStart);
            var endIndex = Math.ceil(timeDifferenceEnd - eps);

            var displayEvent = {
              event: event,
              startIndex: startIndex,
              endIndex: endIndex
            };

            eventSet = rows[startIndex].events;
            if (eventSet) {
              eventSet.push(displayEvent);
            } else {
              eventSet = [];
              eventSet.push(displayEvent);
              rows[startIndex].events = eventSet;
            }
          }
        }

        if (normalEventInRange) {
          var orderedEvents = [];
          for (hour = 0; hour < 24; hour += 1) {
            if (rows[hour].events) {
              orderedEvents = orderedEvents.concat(rows[hour].events);
            }
          }
          if (orderedEvents.length > 0) {
            rows.hasEvent = true;
            ctrl.placeEvents(orderedEvents);
          }
        }

        scope.allDayEvents = allDayEvents;

        $timeout(function () {
          updateScrollGutter();
        });
      };

      ctrl._refreshView = function () {
        var startingDate = ctrl.range.startTime;

        scope.rows = createDateObjects(startingDate);
        scope.allDayEvents = [];
        scope.dates = [startingDate];
        scope.$parent.title = dateFilter(startingDate, ctrl.formatDayTitle);
      };

      ctrl._getRange = function getRange(currentDate) {
        var year = currentDate.getFullYear(),
          month = currentDate.getMonth(),
          date = currentDate.getDate(),
          startTime = new Date(year, month, date),
          endTime = new Date(year, month, date + 1);

        return {
          startTime: startTime,
          endTime: endTime
        };
      };

      ctrl.refreshView();
    }
  };
}]);
